from setuptools import setup

setup(name='jtjjdata_alpha',
      version='1.4.2',
      packages=['DataLoader'],
      maintainer='wcc',
      maintainer_email='wangcc@csc.com.cn',
      url="https://github.com/jimmy")
